/** 
 * Rappresenta i criteri di ricerca.
 * Un criterio di ricerca e' costituito da requisiti su una persona
 * (titolare di abbonamento), e requisiti sul suo abbonamento
 * (titolo posseduto dalla persona), memorizzati negli attributi
 * ereditati dalla super-classe InfoBase.
 * <BR>
 * Nell'attributo categoria dell'abbonamento riesco a memorizzare
 * valori multipli sfruttando il fatto che Le costanti che li
 * rappresentano sono numeri interi che in rappresentazione binaria
 * hanno un solo 1, in posizioni diverse tra loro.
 * I valori muklipli si ottengono facendo OR bit a bit sulla 
 * rappresentazione binaria, es. categ = STUDENTI | ANZIANI
 * (ottengo un numero che ha due 1: dove uno o l'altro aveva 1).
 * Per vedere se due valori multipli hanno intersezione non vuota,
 * facciamo AND bit a bit, es. categ1 & categ2, e guardiamo se e' non zero
 * (ottengo un numero che ha 1 dove entrambi avevano 1).
 */
public class Criterio extends InfoBase
{

  // Costruttori
  
  /**
   * Crea criterio dove tutti gli attrinuti della persona e 
   * dell'abbonamento sono valori non specificati ("" le stringhe e
   * 0 i numeri). Questo criterio e' soddisfatto da qualsiasi abbonato.
   */    
  public Criterio() {  super();  }

  /** Costruttore di copia. */
  public Criterio(InfoBase info)
  {
    titolare = new Persona(info.titolare);
    titolo = new Abbonamento(info.titolo);
  }
  
  // Funzione ausiliaria per corrispondenza codici categoria
  
  /**
   * Controlla la categoria dell'abbonamento presente in questo criterio
   * corrisponde alla categoria data. Entrambi espressi in codice 
   * numerico. Ritorna true se uno e' indefinito o la combinazione in 
   * AND dei due e' non zero (cioe' i due hanno un bit a 1 in comune).
   * @param  codice  il codice di categoria da confrontare con
   *                 quello memorizzato in questo criterio
   */
  public boolean categoriaCorrisponde(int codice)
  {
    // indefinito corrisponde a qualsiasi codice
    if (titolo.categoria==0) return true;
    // se non hanno bit in comune non corrispondono
    if ((titolo.categoria & codice)==0) return false;
    // hanno bit in comune quindi corrispondono
    return true;
  }
  
  // Funzione pubblica per controllare corrispondenza al criterio
  
  /** 
   * Controlla se l'abbonato specificato soddisfa questo criterio
   * di ricerca. Lo soddisfa se, per ogni campo sia della persona 
   * titolare sia dell'abbonamento, o questo criterio non 
   * specifica un valore, oppure il campo ha lo stesso contenuto
   * in questo criterio e nell'abbonato.<BR>
   * Unica eccezione il campo categoria, che nel criterio puo'
   * contenere valori multipli: qui e' rischiesto che l'abbonato
   * abbia uno dei valori.
   * @param   abb    abbonato da controllare
   * @return  true se l'abbonato soddisfa il criterio, false altrimenti
   */
  public boolean corrisponde(Abbonato abb)
  {
    // se il cognome  e' richiesto e non corrisponde
    if ( (titolare.getCognome().length()>0) && 
         (titolare.getCognome().compareTo(abb.titolare.getCognome())!=0) )
       return false;
    // se il nome e' richiesto e non corrisponde
    if ( (titolare.getNome().length()>0) && 
         (titolare.getNome().compareTo(abb.titolare.getNome())!=0) )
       return false;
    // se l'anno di nascita e' richiesto e non corrisponde   
    if ( (titolare.getAnnoNascita()!=0) &&
         (titolare.getAnnoNascita()!=abb.titolare.getAnnoNascita()) )
       return false;
    // se la categoria di abbonamento non corrisponde
    if (!categoriaCorrisponde(abb.titolo.categoria))
       return false;
    // se la durata dell'abbonamento non corrisponde
    if ( (titolo.durata!=0) && (titolo.durata != abb.titolo.durata) )
       return false;
    // se invece ho passato tutti i controlli fatti 
    return true;
  }
}

