import java.io.*;

/**
 * Realizza la base di dati per gli abbonati.<BR>
 * Su disco il data base e' una directory che contiene i file
 * con le informazioni sugli abbonati.
 * Ogni abbonato e' memorizzato in un file con estensione ".abb",
 * mentre il nome file e' dato dalla concatenazione cognome + nome 
 * + data di nascita dell'abbonato.<BR>
 * Implementa FileFilter (ved. funzione accept) in modo tale da 
 * riuscire a filtrare solo i file il cui nome ha estensione ".abb".<BR>
 * Il costruttore prende in argomento il nome della directory e
 * controlla che esista.
 * Successivamente guardandovi dentro posso ricavare:
 * quanti abbonati ci sono, i nomi dei file.<BR>
 * Fornisce anche la funzione per ricercare e caricare
 * dal data base tutti e soli gli abbonati
 * che corrispondono a un dato criterio di ricerca.
 */
public class DataBase implements FilenameFilter
{

  // Attributi
  
  /** Estensione che deve avere un file gita per essere riconosciuto. */
  public static final String ESTENSIONE= ".abb";

  /** Nome della directory contenente gli abbonati del data base. */
  protected String dir;

  // Funzioni definite per implementare l'interfaccia FileFilter

  /** Filtra solo i file che hanno estensione ESTENSIONE.
   * Implementa il metodo dichiarato in FileNameFilter. */ 
  public boolean accept(File d, String nome)
  {
    if (nome.endsWith(ESTENSIONE)) return true;
    return false;
  }

  // Costruttore
  
  /** 
   * Costruisce data base che conterra' tutti gli abbonati nei file
   * di estensione ESTENSIONE presenti nella directory indicata.
   * Lancia eccezione se il nome dato non e' una directory o non esiste.
   * @param  nomeDir   nome della directory contenente i file del
   *                   data base
   */
  public DataBase(String nomeDir) throws Exception
  {
    File f = new File(nomeDir);
    if (!f.exists())
      throw new Exception("DataBase: file " + nomeDir + " non esiste");
    if (!f.isDirectory()) 
      throw new Exception("DataBase: " + nomeDir + " non e' directory");
    dir = nomeDir;
  }

  // Funzioni che ritornano informazioni sul contenuto del data base
  
  /** Restituisce il numero totale di abbonati presenti nel data base.
   * Lo ottiene facendo la lista dei nomi dei file e filtrando solo
   * quelli con estensione ".abb". 
   */
  public int numeroAbbonati() 
  { 
    File f = new File(dir);
    // in costruzione abbiamo gia' verificato che f.exists()
    // e f.isDirectory(), ora dal contenuto della directory prendo
    // i nomi file che soddisfano questo file filter, cioe'
    // quelli con estensione ".abb" e li conto
    String[] nomi = f.list(this);
    if (nomi==null) return 0;
    else return nomi.length;
  }

  // Funzioni per caricare e salvare abbonati nel data base
  
  /**
   * Carica dal data base gli abbonati che soddisfano il criterio di
   * ricerca assegnato, e li ritorna in un array.
   * @param  ric  il criterio di ricerca che deve essere soddisfatto
   * @see Criterio#corrisponde */
  public Abbonato[] carica(Criterio ric) throws Exception
  {
    File f = new File(dir);
    // in costruzione abbiamo gia' verificato che f.exists()
    // e f.isDirectory()...
    String[] nomi = f.list(this);
    int i;
    Abbonato abb;
    int numeroRisultati = 0;
    // prima li mette in array provvisorio troppo grande e li conta
    Abbonato[] aux = new Abbonato[nomi.length];
//System.out.println("numero abbonati="+nomi.length);
    // li legge tutti e memorizza solo quelli che corrispondono
    // al criterio
    for (i=0; i<nomi.length; i++)
    {
//System.out.println("carico da "+dir+File.separatorChar+nomi[i]);
      abb = LeggiScriviAbbonati.leggi(dir + File.separatorChar + nomi[i]);
      if (ric.corrisponde(abb))
      {
        aux[numeroRisultati] = abb;
        numeroRisultati++;
      }
    }
//System.out.println("numero risultati="+numeroRisultati);
    // poi dimensiona l'array finale e vi travasa le gite caricate
    Abbonato[] risultato = new Abbonato[numeroRisultati];
    for (i=0; i<numeroRisultati; i++)
    {
      risultato[i] = aux[i];
    } 
    return risultato;   
  }

  /**
   * Salva su file, nel data base, l'abbonamento specificato.
   * @param abb   l'abbonamento da salvare
   * @see DataBase#nomeFile
   */
  public void salva(Abbonato abb) throws Exception
  {  LeggiScriviAbbonati.scrivi(abb,nomeFile(abb));  }
  
  // Funzioni per gestione dei nomi dei file
  
  /**
   * Costruisce il nome file di un abbonamento dove il titolare
   * ha cognome, nome e anno di nascita specificati.
   * @param  c   cognome del titolare
   * @param  n   nome del titolare
   * @param  a   anno di nascita del titolare
   * @return   nome del file dove memorizzare l'abbonato,
   *           ottenuto concatenando c+n+a con estensione ".abb"
   */ 
  public String nomeFile(String c, String n, int a)
  {  return (dir + File.separatorChar + c + n + a + ESTENSIONE);  }
  
  /**
   * Costruisce il nome file di un abbonato specificato,
   * viene costruito in base a cognome, nome e anno di nascita
   * del titolare.
   * @param  a   l'abbonato 
   * @return     nome del file dove memorizzare l'abbonato,
   *           ottenuto concatenando cognome+nome+aanno di 
   *           di nascita del titolare con estensione ".abb"
   */ 
  public String nomeFile(Abbonato a)
  {
     return nomeFile(a.titolare.getCognome(), a.titolare.getNome(),
            a.titolare.getAnnoNascita());
  }

}
